(******************************************************************************
 * TACLinkCreate                                                              *
 *                                                                            *
 * Because CreateLink calls the CoCreateInstance function, it is assumed      *
 * that the CoInitialize function has already been called. CreateLink uses    *
 * the IPersistFile interface to save the shortcut and the IShellLink         *
 * interface to store the filename and description.                           *
 *                                                                            *
 * Before:                                                                    *
 *	   CoInitialize(nil)                                                  *
 *                                                                            *
 * After:                                                                     *
 *         CoUnInitialize                                                     *
 *                                                                            *
 ******************************************************************************)
unit ACLinkCreate;
(******************************************************************************
 * Interface                                                                  *
 ******************************************************************************)
interface

(******************************************************************************
 * Uses                                                                       *
 ******************************************************************************)
uses
  {$IfDef Ver90}
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;
  {$Else}
  Windows, Classes, ShlObj, ShellApi, SysUtils, ComObj, ComCtrls, StdCtrls, ActiveX, Registry;
  {$EndIf}

(******************************************************************************
 * TAcCreateLink                                                              *
 ******************************************************************************)
type
  TACCreateLink = class(TComponent)
  private
    { Private declarations }
      fTarget:string;
      fDescription:string;
      fWorkingDir:string;
      fArguments:string;
      fHotKey:Integer;
      fShowCmd:Integer;
      fIconLocation:String;
      fIconNumber:Integer;
      fLinkName:String;
      FLinkOnDesktop:Boolean;
      FDeskTopFileName:String;
      FDirectoryForWork:String;
  protected
    { Protected declarations }
  public
    { Public declarations }
    Function Execute:Boolean;
    {return full name link on desktop, folder is getting from registry}
    Function GetDesktopLink:String;
    Function Read:Boolean;
    {setlinkname use before call a read method for right filename of link setting}
    Procedure SetLinkName(Const Filename:String);
  published
    { Published declarations }
    Property Target : String Read ftarget Write ftarget;
    Property Description : String Read fDescription Write fDescription;
    Property LinkDir : String Read fWorkingDir Write fWorkingDir;
    Property Arguments : String Read fArguments Write fArguments;
    Property HotKey : Integer Read fHotKey Write fHotKey;
    Property ShowCmd : Integer Read fShowCmd Write fShowCmd;
    Property IconLocation : String Read fIconLocation Write fIconLocation;
    Property IconNumber : Integer Read fIconNumber Write fIconNumber;
    Property LinkName : String Read fLinkName Write fLinkName;
    Property LinkOnDesktop : Boolean Read FLinkOnDesktop Write FLinkOnDesktop;
    Property DirectoryForWork :String Read FDirectoryForWork Write FDirectoryForWork;
  end;

(******************************************************************************
 * Implementation                                                             *
 ******************************************************************************)
implementation

(******************************************************************************
 * Uses                                                                       *
 ******************************************************************************)
{$IfDef Ver90} Uses ole2, ShlObj; {$EndIf}

(******************************************************************************
 * Execute                                                                    *
 *                                                                            *
 * Ritorna                                                                    *
 *   True: Ok                                                                 *
 *   False: Errore                                                            *
 ******************************************************************************)
Function TACCreateLink.Execute:Boolean;
{$IfDef VER90}
var Txt1,Txt2 : array[0..255] of char;
    Ris       : HResult;
    Psl       : IShellLink;
    Ppf       : IPersistFile;
    Saver     : Array [0..Max_Path] of WideChar;

begin
  if (CoCreateInstance(CLSID_ShellLink,
                       nil,
                       CLSCTX_INPROC_SERVER,
                       IID_IShellLink,psl)<>0) then
  begin
    Execute := False;
    Exit;
  end;

  try
    StrPCopy(txt1,Target);
    if (psl.SetPath(PChar(fTarget))<>0) then
    begin
      Execute := False;
      Exit;
    end;

    StrPCopy(txt1,Description);
    Ris := 0;
    Ris := Ris + psl.SetDescription(txt1);
    Ris := Ris + psl.SetWorkingDirectory(PChar(DirectoryForWork));
    Ris := Ris + psl.SetArguments(PChar(Arguments));
    Ris := Ris + psl.SetHotKey(HotKey);
    Ris := Ris + psl.SetShowCmd(ShowCmd);
    Ris := Ris + psl.SetIconLocation(PChar(IconLocation),IconNumber);
    If Ris<>0 then
    begin
      Execute:=False;
      Exit;
    end;


    if (psl.QueryInterface(IID_IPersistFile,ppf)<>0) then
    begin
      Execute := False;
      Exit;
    end;

    StrPCopy(txt2,LinkName);

    MultiByteToWideChar(CP_ACP,0,txt2,-1,Saver,Max_Path);

    if (ppf.Save(Saver,True)<>0) then
    begin
      Execute := False;
      Exit;
    end;
    ppf.release;
  finally
    psl.release;
  end;
  Execute := True;
{$Else}
  Function CreateLnk(lnkName, lnkObject, lnkDir, lnkParam, lnkWorkDir,
    lnkDescription,iconFileName: String;
    IconIndex, lnkHotKey,lnkShowCmd:Integer; Desktop: Boolean):String;
    //universal procedure for setting a .lnk data
  Const
    MSDskPath='Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders';
  var RegData: TRegistry;
    function UpdatePath(sPath: String): String;
    begin
      If Copy(sPath, Length(sPath), 1) <> '\' Then Result:= Concat(sPath, '\')
      Else Result:= sPath;
    end;
  var
    NewObject: IUnknown;
    NewLink: IShellLink;
    MyPFile: IPersistFile;
    sDesktop: String;
    WFileName: WideString;
  begin
    Result:='';
    NewObject:= CreateComObject(CLSID_ShellLink);
    NewLink:= NewObject as IShellLink;
    MyPFile:= NewObject as IPersistFile;
    With NewLink do
    begin
      SetArguments(PChar(lnkParam));
      SetPath(PChar(lnkObject));
      If lnkWorkDir <> '' Then
        SetWorkingDirectory(PChar(lnkWorkDir)) {another directory}
      Else
        SetWorkingDirectory(PChar(ExtractFilePath(lnkObject))); {associated file directory}
      If iconFileName <> '' Then
        SetIconLocation(PChar(iconFileName),IconIndex){from other file}
      Else
        SetIconLocation(PChar(lnkObject),IconIndex);{from associated file}
      SetDescription(PChar(lnkDescription));
      SetHotKey(lnkHotKey);
      SetShowCmd(lnkShowCmd);
    end;
    sDesktop:='';
    RegData:= TRegistry.Create;
    Try
      With RegData do
      begin
        RootKey:= HKEY_CURRENT_USER;
        If OpenKey(MSDskPath, False) Then //link on desktop, for easy use without copiing
          sDesktop:= UpdatePath(ReadString('Desktop'));
      end;
    Finally
      RegData.Free;
    End;

    If Desktop Then
      WFileName:= Concat(sDesktop, lnkName, '.lnk')
    Else
      WFileName:= Concat(UpdatePath(lnkDir), lnkName, '.lnk');
    Result := WFileName;
    MyPFile.Save(PWChar(WFileName), False);

    NewObject._Release;
  end;
Begin
  FDeskTopFileName := CreateLnk(
    LinkName, {name of .LNK}
    Target, {}
    linkDir,
    Arguments,
    DirectoryForWork,
    Description,
    IconLocation,
    IconNumber,
    HotKey,
    ShowCmd,
    LinkOnDeskTop);
{$EndIf}
End;

Function TACCreateLink.GetDeskTopLink:String;
Begin
  Result := '';
  If FDeskTopFileName <> '' Then
    Result := FDeskTopFileName;
End;

Procedure TACCreateLink.SetLinkName(Const Filename:String);
Begin
  fWorkingDir := ExtractFilePath(Filename);
  FLinkName := ExtractFileName(Filename);
End;

(******************************************************************************
 * Read                                                                       *
 ******************************************************************************)
Function TACCreateLink.Read:Boolean;
{$IfDef Ver90}
var Txt1      : Array[0..255] of char;
    Ris       : HResult;
    Psl       : IShellLink;
    Ppf       : IPersistFile;
    Saver     : Array [0..Max_Path] of WideChar;
    X1        : Array [0..255] Of Char;
    Data      : TWin32FindData;
    App1      : Integer;
    W         : Word;
begin
  if (CoCreateInstance(CLSID_ShellLink,
                      nil,
                      CLSCTX_INPROC_SERVER,
                      IID_IShellLink,psl)<>0) Then
  begin
    Read:=False;
    Exit;
  end;
  if (psl.QueryInterface(IID_IPersistFile,ppf)<>0) Then
  begin
    Read:=False;
    Exit;
  end;
  StrPCopy(Txt1,LinkName);
  MultiByteToWideChar(CP_ACP,0,Txt1,-1,Saver,Max_Path);
  if (ppf.Load(Saver,STGM_READ)<>0) then
  begin
    Read:=False;
    Exit;
  end;

  if (psl.Resolve(0,SLR_ANY_MATCH)<>0) then
  begin
    Read:=False;
    Exit;
  end;

  if (psl.GetWorkingDirectory(@X1,MAX_PATH )<>0) then
  begin
    Read:=False;
    Exit;
  end;

  fWorkingDir  := StrPas(@X1);
  Ris          := psl.GetPath( @X1,MAX_PATH,Data,SLGP_UNCPRIORITY);
  Target       := StrPas(@X1);
  Ris          := Ris + psl.GetIconLocation(@X1,MAX_PATH,App1);
  IconLocation := StrPas(@X1);
  IconNumber   := App1;
  Ris          := Ris + psl.GetDescription(@X1,MAX_PATH );
  Description  := StrPas(@X1);
  App1         := 0;
  Ris          := Ris + psl.GetShowCmd(App1);
  fShowCmd     := App1;
  W            := 0;
  Ris          := Ris + psl.GetHotKey(W);
  fHotKey      := W;
  Ris          := Ris + psl.GetArguments(@X1,MAX_PATH );
  fArguments   := StrPas(@X1);

  ppf.release;
  psl.release;
  Read := (Ris = 0);
{$Else}
Const pLen = 512;
var
    NewObject: IUnknown;
    NewLink: IShellLink;
    MyPFile: IPersistFile;
    sDesktop: String;
    WFileName: WideString;
    pStr:String;
    I:Integer;
    W:Word;
    data:TWin32FindData;
Begin
    Result := False;
    NewObject := CreateComObject(CLSID_ShellLink);
    NewLink := NewObject as IShellLink;
    MyPFile := NewObject as IPersistFile;
    WFileName:= LinkDir+LinkName+#0#0;
    If MyPFile.Load(@WFileName[1],STGM_READ) = S_OK Then With NewLink do
    Begin
      SetLength(pStr,pLen);
      GetArguments(PChar(pStr),plen-1);
      Arguments := Trim(pStr); {have to trimmed because #0 added to end sometimes}
      SetLength(pStr,pLen);
      GetPath(PChar(pStr),plen-1,data,SLGP_UNCPRIORITY);
      LinkDir := Trim(ExtractFilePath(pStr));
      SetLength(pStr,pLen);
      GetWorkingDirectory(PChar(pStr),plen-1);
      DirectoryForWork := Trim(pStr);
      SetLength(pStr,pLen);
      GetIconLocation(PChar(pStr),plen-1,I);
      IconLocation := Trim(pStr);
      IconNumber := I;
      SetLength(pStr,pLen);
      GetDescription(PChar(pStr),plen-1);
      Description := Trim(pStr);
      GetHotKey(W);
      HotKey := W;
      GetShowCmd(I);
      ShowCmd := I;
      Result := True; {if all OK then get true result}
    End;
{$EndIf}
end;

end.
